@@ -6,6 +6,6 @@
 // SPDX-License-Identifier: BSD-3-Clause
 
 
-#include <config.h>
+#include "config.h"
 
 #include "alloc/realloc.h"
