@@ -6,7 +6,7 @@
 #define SHADOW_INCLUDE_LIB_CAST_H_
 
 
-#include <config.h>
+#include "config.h"
 
 
 #define const_cast(T, p)  _Generic(p, const T:  (T) (p))
