@@ -35,39 +35,30 @@ struct commonio_ops {
 	 * Make a copy of the object (for example, struct passwd)
 	 * and all strings pointed by it, in malloced memory.
 	 */
-	/*@null@*/ /*@only@*/void *(*dup) (const void *);
+	/*@null@*/ /*@only@*/void *(*cio_dup)(const void *);
 
 	/*
 	 * free() the object including any strings pointed by it.
 	 */
-	void (*free)(/*@only@*/void *);
+	void (*cio_free)(/*@only@*/void *);
 
 	/*
 	 * Return the name of the object (for example, pw_name
 	 * for struct passwd).
 	 */
-	const char *(*getname) (const void *);
+	const char *(*cio_getname)(const void *);
 
 	/*
 	 * Parse a string, return object (in static area -
 	 * should be copied using the dup operation above).
 	 */
-	void *(*parse) (const char *);
+	void *(*cio_parse)(const char *);
 
 	/*
 	 * Write the object to the file (this calls putpwent()
 	 * for struct passwd, for example).
 	 */
-	int (*put) (const void *, FILE *);
-
-	/*
-	 * fgets and fputs (can be replaced by versions that
-	 * understand line continuation conventions).
-	 */
-	ATTR_ACCESS(write_only, 1, 2)
-	/*@null@*/char *(*fgets)(/*@returned@*/char *restrict s, int n,
-	                         FILE *restrict stream);
-	int (*fputs) (const char *, FILE *);
+	int (*cio_put)(const void *, FILE *);
 
 	/*
 	 * open_hook and close_hook.
@@ -75,8 +66,8 @@ struct commonio_ops {
 	 * is open or before it is closed.
 	 * They return 0 on failure and 1 on success.
 	 */
-	/*@null@*/int (*open_hook) (void);
-	/*@null@*/int (*close_hook) (void);
+	/*@null@*/int (*cio_open_hook)(void);
+	/*@null@*/int (*cio_close_hook)(void);
 };
 
 /*
@@ -103,7 +94,7 @@ struct commonio_db {
 #endif
 	/*
 	 * Default permissions and owner for newly created data file.
-         */
+	 */
 	mode_t st_mode;
 	uid_t st_uid;
 	gid_t st_gid;
@@ -138,8 +129,8 @@ extern int commonio_append (struct commonio_db *, const void *);
 extern int commonio_remove (struct commonio_db *, const char *);
 extern int commonio_rewind (struct commonio_db *);
 extern /*@observer@*/ /*@null@*/const void *commonio_next (struct commonio_db *);
-extern int commonio_close (struct commonio_db *);
-extern int commonio_unlock (struct commonio_db *);
+extern int commonio_close (struct commonio_db *, bool);
+extern int commonio_unlock (struct commonio_db *, bool);
 extern void commonio_del_entry (struct commonio_db *,
                                 const struct commonio_entry *);
 extern int commonio_sort_wrt (struct commonio_db *shadow,
