@@ -8,7 +8,7 @@
  * SPDX-License-Identifier: BSD-3-Clause
  */
 
-#include <config.h>
+#include "config.h"
 
 #ident "$Id$"
 
@@ -25,7 +25,7 @@
 	struct group *gr;
 	int i;
 
-	gr = CALLOC(1, struct group);
+	gr = calloc_T(1, struct group);
 	if (NULL == gr) {
 		return NULL;
 	}
@@ -49,7 +49,7 @@
 	for (i = 0; grent->gr_mem[i]; i++);
 
 	/*@-mustfreeonly@*/
-	gr->gr_mem = MALLOC(i + 1, char *);
+	gr->gr_mem = malloc_T(i + 1, char *);
 	/*@=mustfreeonly@*/
 	if (NULL == gr->gr_mem) {
 		gr_free(gr);
