@@ -11,7 +11,7 @@
  *
  */
 
-#include <config.h>
+#include "config.h"
 
 #ident "$Id$"
 
