@@ -6,7 +6,7 @@
 #define SHADOW_INCLUDE_LIB_STRING_STRSPN_STPSPN_H_
 
 
-#include <config.h>
+#include "config.h"
 
 #include <string.h>
 
@@ -19,7 +19,7 @@
 #define stpspn(s, accept)                                                     \
 ({                                                                            \
 	__auto_type  s_ = s;                                                  \
-                                                                              \
+	                                                                      \
 	s_ + strspn(s_, accept);                                              \
 })
 
