@@ -7,7 +7,7 @@
  * SPDX-License-Identifier: BSD-3-Clause
  */
 
-#include <config.h>
+#include "config.h"
 
 #ident "$Id$"
 
@@ -34,7 +34,7 @@
 #include "sssd.h"
 #include "string/strcmp/streq.h"
 #include "string/strcpy/strtcpy.h"
-#include "string/strdup/xstrdup.h"
+#include "string/strdup/strdup.h"
 
 
 #ifndef SHELLS_FILE
@@ -47,6 +47,9 @@
 #define ETCDIR "/etc"
 #endif
 
+struct option_flags {
+	bool chroot;
+};
 
 /*
  * Global variables
@@ -61,24 +64,25 @@ static bool pw_locked = false;
 /* external identifiers */
 
 /* local function prototypes */
-NORETURN static void fail_exit (int code);
+NORETURN static void fail_exit (int code, bool process_selinux);
 NORETURN static void usage (int status);
 static void new_fields (void);
-static bool shell_is_listed (const char *);
-static bool is_restricted_shell (const char *);
-static void process_flags (int argc, char **argv);
-static void check_perms (const struct passwd *pw);
-static void update_shell (const char *user, char *loginsh);
+static bool shell_is_listed (const char *, bool process_selinux);
+static bool is_restricted_shell (const char *, bool process_selinux);
+static void process_flags (int argc, char **argv, struct option_flags *flags);
+static void check_perms(const struct passwd *pw, const struct option_flags *flags);
+static void update_shell (const char *user, char *loginsh,
+                          const struct option_flags *flags);
 
 /*
  * fail_exit - do some cleanup and exit with the given error code
  */
 NORETURN
 static void
-fail_exit (int code)
+fail_exit (int code, bool process_selinux)
 {
 	if (pw_locked) {
-		if (pw_unlock () == 0) {
+		if (pw_unlock (process_selinux) == 0) {
 			fprintf (stderr, _("%s: failed to unlock %s\n"), Prog, pw_dbname ());
 			SYSLOG ((LOG_ERR, "failed to unlock %s", pw_dbname ()));
 			/* continue */
@@ -119,21 +123,21 @@ usage (int status)
 static void new_fields (void)
 {
 	puts (_("Enter the new value, or press ENTER for the default"));
-	change_field (loginsh, sizeof loginsh, _("Login Shell"));
+	change_field(loginsh, sizeof(loginsh), _("Login Shell"));
 }
 
 /*
  * is_restricted_shell - return true if the shell is restricted
  *
  */
-static bool is_restricted_shell (const char *sh)
+static bool is_restricted_shell (const char *sh, bool process_selinux)
 {
 	/*
 	 * Shells not listed in /etc/shells are considered to be restricted.
 	 * Changed this to avoid confusion with "rc" (the plan9 shell - not
 	 * restricted despite the name starting with 'r').  --marekm
 	 */
-	return !shell_is_listed (sh);
+	return !shell_is_listed (sh, process_selinux);
 }
 
 /*
@@ -148,7 +152,7 @@ static bool is_restricted_shell (const char *sh)
  */
 
 #ifdef HAVE_VENDORDIR
-static bool shell_is_listed (const char *sh)
+static bool shell_is_listed (const char *sh, bool process_selinux)
 {
 	bool found = false;
 
@@ -168,7 +172,7 @@ static bool shell_is_listed (const char *sh)
 		fprintf (stderr,
 			 _("Cannot parse shell files: %s"),
 			 econf_errString(error));
-		fail_exit (1);
+		fail_exit (1, process_selinux);
 	}
 
 	error = econf_getKeys(key_file, NULL, &size, &keys);
@@ -177,7 +181,7 @@ static bool shell_is_listed (const char *sh)
 			 _("Cannot evaluate entries in shell files: %s"),
 			 econf_errString(error));
 		econf_free (key_file);
-		fail_exit (1);
+		fail_exit (1, process_selinux);
 	}
 
 	for (size_t i = 0; i < size; i++) {
@@ -194,7 +198,7 @@ static bool shell_is_listed (const char *sh)
 
 #else /* without HAVE_VENDORDIR */
 
-static bool shell_is_listed (const char *sh)
+static bool shell_is_listed (const char *sh, bool)
 {
 	bool found = false;
 	char *cp;
@@ -217,7 +221,7 @@ static bool shell_is_listed (const char *sh)
  *
  *	It will not return if an error is encountered.
  */
-static void process_flags (int argc, char **argv)
+static void process_flags (int argc, char **argv, struct option_flags *flags)
 {
 	int c;
 	static struct option long_options[] = {
@@ -234,10 +238,11 @@ static void process_flags (int argc, char **argv)
 			usage (E_SUCCESS);
 			/*@notreached@*/break;
 		case 'R': /* no-op, handled in process_root_flag () */
+			flags->chroot = true;
 			break;
 		case 's':
 			sflg = true;
-			STRTCPY(loginsh, optarg);
+			strtcpy_a(loginsh, optarg);
 			break;
 		default:
 			usage (E_USAGE);
@@ -263,13 +268,16 @@ static void process_flags (int argc, char **argv)
  *
  *	It will not return if the user is not allowed.
  */
-static void check_perms (const struct passwd *pw)
+static void check_perms(const struct passwd *pw, const struct option_flags *flags)
 {
 #ifdef USE_PAM
 	pam_handle_t *pamh = NULL;
 	int retval;
 	struct passwd *pampw;
 #endif
+	bool process_selinux;
+
+	process_selinux = !flags->chroot;
 
 	/*
 	 * Non-privileged users are only allowed to change the shell if the
@@ -280,19 +288,19 @@ static void check_perms (const struct passwd *pw)
 		fprintf (stderr,
 		         _("You may not change the shell for '%s'.\n"),
 		         pw->pw_name);
-		fail_exit (1);
+		fail_exit (1, process_selinux);
 	}
 
 	/*
 	 * Non-privileged users are only allowed to change the shell if it
 	 * is not a restricted one.
 	 */
-	if (!amroot && is_restricted_shell (pw->pw_shell)) {
+	if (!amroot && is_restricted_shell (pw->pw_shell, process_selinux)) {
 		SYSLOG ((LOG_WARN, "can't change shell for '%s'", pw->pw_name));
 		fprintf (stderr,
 		         _("You may not change the shell for '%s'.\n"),
 		         pw->pw_name);
-		fail_exit (1);
+		fail_exit (1, process_selinux);
 	}
 #ifdef WITH_SELINUX
 	/*
@@ -305,7 +313,7 @@ static void check_perms (const struct passwd *pw)
 		fprintf (stderr,
 		         _("You may not change the shell for '%s'.\n"),
 		         pw->pw_name);
-		fail_exit (1);
+		fail_exit (1, process_selinux);
 	}
 #endif
 
@@ -317,8 +325,8 @@ static void check_perms (const struct passwd *pw)
 	 * chfn/chsh.  --marekm
 	 */
 	if (!amroot && getdef_bool ("CHSH_AUTH")) {
-		passwd_check (pw->pw_name, pw->pw_passwd, Prog);
-        }
+		passwd_check(pw->pw_name, pw->pw_passwd);
+	}
 
 #else				/* !USE_PAM */
 	pampw = getpwuid (getuid ()); /* local, no need for xgetpwuid */
@@ -359,10 +367,13 @@ static void check_perms (const struct passwd *pw)
  *
  *	It will not return in case of error.
  */
-static void update_shell (const char *user, char *newshell)
+static void update_shell (const char *user, char *newshell, const struct option_flags *flags)
 {
 	const struct passwd *pw;	/* Password entry from /etc/passwd   */
 	struct passwd pwent;		/* New password entry                */
+	bool process_selinux;
+
+	process_selinux = !flags->chroot;
 
 	/*
 	 * Before going any further, raise the ulimit to prevent
@@ -373,7 +384,7 @@ static void update_shell (const char *user, char *newshell)
 	if (setuid (0) != 0) {
 		SYSLOG ((LOG_ERR, "can't setuid(0)"));
 		fputs (_("Cannot change ID to root.\n"), stderr);
-		fail_exit (1);
+		fail_exit (1, process_selinux);
 	}
 	pwd_init ();
 
@@ -384,13 +395,13 @@ static void update_shell (const char *user, char *newshell)
 	if (pw_lock () == 0) {
 		fprintf (stderr, _("%s: cannot lock %s; try again later.\n"),
 		         Prog, pw_dbname ());
-		fail_exit (1);
+		fail_exit (1, process_selinux);
 	}
 	pw_locked = true;
 	if (pw_open (O_CREAT | O_RDWR) == 0) {
 		fprintf (stderr, _("%s: cannot open %s\n"), Prog, pw_dbname ());
 		SYSLOG ((LOG_WARN, "cannot open %s", pw_dbname ()));
-		fail_exit (1);
+		fail_exit (1, process_selinux);
 	}
 
 	/*
@@ -404,7 +415,7 @@ static void update_shell (const char *user, char *newshell)
 		fprintf (stderr,
 		         _("%s: user '%s' does not exist in %s\n"),
 		         Prog, user, pw_dbname ());
-		fail_exit (1);
+		fail_exit (1, process_selinux);
 	}
 
 	/*
@@ -422,18 +433,18 @@ static void update_shell (const char *user, char *newshell)
 		fprintf (stderr,
 		         _("%s: failed to prepare the new %s entry '%s'\n"),
 		         Prog, pw_dbname (), pwent.pw_name);
-		fail_exit (1);
+		fail_exit (1, process_selinux);
 	}
 
 	/*
 	 * Changes have all been made, so commit them and unlock the file.
 	 */
-	if (pw_close () == 0) {
+	if (pw_close (process_selinux) == 0) {
 		fprintf (stderr, _("%s: failure while writing changes to %s\n"), Prog, pw_dbname ());
 		SYSLOG ((LOG_ERR, "failure while writing changes to %s", pw_dbname ()));
-		fail_exit (1);
+		fail_exit (1, process_selinux);
 	}
-	if (pw_unlock () == 0) {
+	if (pw_unlock (process_selinux) == 0) {
 		fprintf (stderr, _("%s: failed to unlock %s\n"), Prog, pw_dbname ());
 		SYSLOG ((LOG_ERR, "failed to unlock %s", pw_dbname ()));
 		/* continue */
@@ -451,6 +462,8 @@ int main (int argc, char **argv)
 {
 	char *user;		/* User name                         */
 	const struct passwd *pw;	/* Password entry from /etc/passwd   */
+	struct option_flags  flags = {.chroot = false};
+	bool process_selinux;
 
 	sanitize_env ();
 	check_fds ();
@@ -472,7 +485,8 @@ int main (int argc, char **argv)
 	OPENLOG (Prog);
 
 	/* parse the command line options */
-	process_flags (argc, argv);
+	process_flags (argc, argv, &flags);
+	process_selinux = !flags.chroot;
 
 	/*
 	 * Get the name of the user to check. It is either the command line
@@ -481,14 +495,14 @@ int main (int argc, char **argv)
 	if (optind < argc) {
 		if (!is_valid_user_name (argv[optind])) {
 			fprintf (stderr, _("%s: Provided user name is not a valid name\n"), Prog);
-			fail_exit (1);
+			fail_exit (1, process_selinux);
 		}
 		user = argv[optind];
 		pw = xgetpwnam (user);
 		if (NULL == pw) {
 			fprintf (stderr,
 			         _("%s: user '%s' does not exist\n"), Prog, user);
-			fail_exit (1);
+			fail_exit (1, process_selinux);
 		}
 	} else {
 		pw = get_my_pwent ();
@@ -498,19 +512,19 @@ int main (int argc, char **argv)
 			         Prog);
 			SYSLOG ((LOG_WARN, "Cannot determine the user name of the caller (UID %lu)",
 			         (unsigned long) getuid ()));
-			fail_exit (1);
+			fail_exit (1, process_selinux);
 		}
 		user = xstrdup (pw->pw_name);
 	}
 
-	check_perms (pw);
+	check_perms (pw, &flags);
 
 	/*
 	 * Now get the login shell. Either get it from the password
 	 * file, or use the value from the command line.
 	 */
 	if (!sflg) {
-		STRTCPY(loginsh, pw->pw_shell);
+		strtcpy_a(loginsh, pw->pw_shell);
 	}
 
 	/*
@@ -530,18 +544,18 @@ int main (int argc, char **argv)
 	 */
 	if (valid_field (loginsh, ":,=\n") != 0) {
 		fprintf (stderr, _("%s: Invalid entry: %s\n"), Prog, loginsh);
-		fail_exit (1);
+		fail_exit (1, process_selinux);
 	}
 	if (!streq(loginsh, "")
 	    && (loginsh[0] != '/'
-	        || is_restricted_shell (loginsh)
+	        || is_restricted_shell (loginsh, process_selinux)
 	        || (access (loginsh, X_OK) != 0)))
 	{
 		if (amroot) {
 			fprintf (stderr, _("%s: Warning: %s is an invalid shell\n"), Prog, loginsh);
 		} else {
 			fprintf (stderr, _("%s: %s is an invalid shell\n"), Prog, loginsh);
-			fail_exit (1);
+			fail_exit (1, process_selinux);
 		}
 	}
 
@@ -555,7 +569,7 @@ int main (int argc, char **argv)
 		}
 	}
 
-	update_shell (user, loginsh);
+	update_shell (user, loginsh, &flags);
 
 	SYSLOG ((LOG_INFO, "changed user '%s' shell to '%s'", user, loginsh));
 
