@@ -7,7 +7,7 @@
  * SPDX-License-Identifier: BSD-3-Clause
  */
 
-#include <config.h>
+#include "config.h"
 
 #ident "$Id$"
 
@@ -26,16 +26,16 @@
 #include "pam_defs.h"
 #endif				/* USE_PAM */
 #endif				/* ACCT_TOOLS_SETUID */
+#include <paths.h>
 #include <stdio.h>
+#include <string.h>
 #include <strings.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <time.h>
 
 #include "alloc/malloc.h"
-#include "alloc/x/xmalloc.h"
-#include "atoi/a2i/a2i.h"
-#include "atoi/a2i/a2s.h"
+#include "atoi/a2i.h"
 #include "atoi/getnum.h"
 #include "chkname.h"
 #include "defines.h"
@@ -59,13 +59,17 @@
 #ifdef WITH_TCB
 #include "tcbfuncs.h"
 #endif
+#include "shadow/gshadow/endsgent.h"
+#include "shadow/gshadow/sgrp.h"
 #include "shadowlog.h"
 #include "sssd.h"
 #include "string/memset/memzero.h"
-#include "string/sprintf/xaprintf.h"
+#include "string/sprintf/aprintf.h"
 #include "string/strcmp/streq.h"
 #include "string/strcmp/strprefix.h"
-#include "string/strdup/xstrdup.h"
+#include "string/strdup/strdup.h"
+#include "string/strerrno.h"
+#include "string/strspn/stprspn.h"
 #include "time/day_to_str.h"
 #include "typetraits.h"
 
@@ -96,6 +100,14 @@
 
 #define VALID(s)  (!strpbrk(s, ":\n"))
 
+/*
+ * Structures
+ */
+struct option_flags {
+	bool chroot;
+	bool prefix;
+};
+
 /*
  * Global variables
  */
@@ -182,23 +194,23 @@ static bool sub_gid_locked = false;
 /* local function prototypes */
 static int get_groups (char *);
 NORETURN static void usage (int status);
-static void new_pwent (struct passwd *);
-static void new_spent (struct spwd *);
-NORETURN static void fail_exit (int);
-static void update_group_file(void);
-static void update_group(const struct group *grp);
+static void new_pwent (struct passwd *, bool);
+static void new_spent (struct spwd *, bool);
+NORETURN static void fail_exit (int, bool);
+static void update_group_file(bool);
+static void update_group(const struct group *grp, bool process_selinux);
 
 #ifdef SHADOWGRP
-static void update_gshadow_file(void);
-static void update_gshadow(const struct sgrp *sgrp);
+static void update_gshadow_file(bool process_selinux);
+static void update_gshadow(const struct sgrp *sgrp, bool process_selinux);
 #endif
-static void grp_update (void);
+static void grp_update (bool process_selinux);
 
-static void process_flags (int, char **);
-static void close_files (void);
-static void open_files (void);
-static void usr_update (void);
-static void move_home (void);
+static void process_flags (int, char **, struct option_flags *);
+static void close_files(const struct option_flags *);
+static void open_files (bool process_selinux);
+static void usr_update(const struct option_flags *flags);
+static void move_home (bool process_selinux);
 #ifdef ENABLE_LASTLOG
 static void update_lastlog (void);
 #endif /* ENABLE_LASTLOG */
@@ -354,11 +366,11 @@ prepend_range(const char *str, struct id_range_list_entry **head)
 	if (range.first > range.last)
 		return 0;
 
-	entry = MALLOC(1, struct id_range_list_entry);
+	entry = malloc_T(1, struct id_range_list_entry);
 	if (!entry) {
 		fprintf (stderr,
 			_("%s: failed to allocate memory: %s\n"),
-			Prog, strerror (errno));
+			Prog, strerrno());
 		return 0;
 	}
 	entry->next = *head;
@@ -429,7 +441,7 @@ static char *new_pw_passwd (char *pw_pass)
 {
 	if (Lflg && ('!' != pw_pass[0])) {
 #ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+		audit_logger (AUDIT_USER_CHAUTHTOK,
 		              "updating-passwd", user_newname, user_newid, 1);
 #endif
 		SYSLOG ((LOG_INFO, "lock user '%s' password", user_newname));
@@ -444,14 +456,14 @@ static char *new_pw_passwd (char *pw_pass)
 		}
 
 #ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+		audit_logger (AUDIT_USER_CHAUTHTOK,
 		              "updating-password", user_newname, user_newid, 1);
 #endif
 		SYSLOG ((LOG_INFO, "unlock user '%s' password", user_newname));
 		memmove(pw_pass, pw_pass + 1, strlen(pw_pass));
 	} else if (pflg) {
 #ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+		audit_logger (AUDIT_USER_CHAUTHTOK,
 		              "updating-password", user_newname, user_newid, 1);
 #endif
 		SYSLOG ((LOG_INFO, "change user '%s' password", user_newname));
@@ -466,7 +478,7 @@ static char *new_pw_passwd (char *pw_pass)
  *	new_pwent() takes all of the values that have been entered and fills
  *	in a (struct passwd) with them.
  */
-static void new_pwent (struct passwd *pwent)
+static void new_pwent (struct passwd *pwent, bool process_selinux)
 {
 	if (lflg) {
 		if (pw_locate (user_newname) != NULL) {
@@ -477,10 +489,10 @@ static void new_pwent (struct passwd *pwent)
 			fprintf (stderr,
 			         _("%s: user '%s' already exists in %s\n"),
 			         Prog, user_newname, pw_dbname ());
-			fail_exit (E_NAME_IN_USE);
+			fail_exit (E_NAME_IN_USE, process_selinux);
 		}
 #ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_MGMT, Prog,
+		audit_logger (AUDIT_USER_MGMT,
 		              "changing-name", user_newname, user_newid, 1);
 #endif
 		SYSLOG ((LOG_INFO,
@@ -500,7 +512,7 @@ static void new_pwent (struct passwd *pwent)
 
 	if (uflg) {
 #ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_MGMT, Prog,
+		audit_logger (AUDIT_USER_MGMT,
 		              "changing-uid", user_newname, user_newid, 1);
 #endif
 		SYSLOG ((LOG_INFO,
@@ -510,7 +522,7 @@ static void new_pwent (struct passwd *pwent)
 	}
 	if (gflg) {
 #ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_MGMT, Prog,
+		audit_logger (AUDIT_USER_MGMT,
 		              "changing-primary-group",
 		              user_newname, user_newid, 1);
 #endif
@@ -521,7 +533,7 @@ static void new_pwent (struct passwd *pwent)
 	}
 	if (cflg) {
 #ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_MGMT, Prog,
+		audit_logger (AUDIT_USER_MGMT,
 		              "changing-comment", user_newname, user_newid, 1);
 #endif
 		pwent->pw_gecos = user_newcomment;
@@ -529,7 +541,7 @@ static void new_pwent (struct passwd *pwent)
 
 	if (dflg) {
 #ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_MGMT, Prog,
+		audit_logger (AUDIT_USER_MGMT,
 		              "changing-home-dir",
 		              user_newname, user_newid, 1);
 #endif
@@ -537,16 +549,14 @@ static void new_pwent (struct passwd *pwent)
 		         "change user '%s' home from '%s' to '%s'",
 		         pwent->pw_name, pwent->pw_dir, user_newhome));
 
-		if (strlen(user_newhome) > 1
-			&& '/' == user_newhome[strlen(user_newhome)-1]) {
-			user_newhome[strlen(user_newhome)-1]='\0';
-		}
+		if (!streq(user_newhome, ""))
+			stpcpy(stprspn(user_newhome + 1, "/"), "");
 
 		pwent->pw_dir = user_newhome;
 	}
 	if (sflg) {
 #ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_MGMT, Prog,
+		audit_logger (AUDIT_USER_MGMT,
 		              "changing-shell",
 		              user_newname, user_newid, 1);
 #endif
@@ -563,21 +573,21 @@ static void new_pwent (struct passwd *pwent)
  *	new_spent() takes all of the values that have been entered and fills
  *	in a (struct spwd) with them.
  */
-static void new_spent (struct spwd *spent)
+static void new_spent (struct spwd *spent, bool process_selinux)
 {
 	if (lflg) {
 		if (spw_locate (user_newname) != NULL) {
 			fprintf (stderr,
 			         _("%s: user '%s' already exists in %s\n"),
 			         Prog, user_newname, spw_dbname ());
-			fail_exit (E_NAME_IN_USE);
+			fail_exit (E_NAME_IN_USE, process_selinux);
 		}
 		spent->sp_namp = xstrdup (user_newname);
 	}
 
 	if (fflg) {
 #ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_MGMT, Prog,
+		audit_logger (AUDIT_USER_MGMT,
 		              "changing-inactive-days",
 		              user_newname, user_newid, 1);
 #endif
@@ -590,10 +600,10 @@ static void new_spent (struct spwd *spent)
 		/* log dates rather than numbers of days. */
 		char new_exp[16], old_exp[16];
 
-		DAY_TO_STR(new_exp, user_newexpire);
-		DAY_TO_STR(old_exp, user_expire);
+		day_to_str_a(new_exp, user_newexpire);
+		day_to_str_a(old_exp, user_expire);
 #ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_MGMT, Prog,
+		audit_logger (AUDIT_USER_MGMT,
 		              "changing-expiration-date",
 		              user_newname, user_newid, 1);
 #endif
@@ -629,10 +639,10 @@ static void new_spent (struct spwd *spent)
  */
 NORETURN
 static void
-fail_exit (int code)
+fail_exit (int code, bool process_selinux)
 {
 	if (gr_locked) {
-		if (gr_unlock () == 0) {
+		if (gr_unlock (process_selinux) == 0) {
 			fprintf (stderr, _("%s: failed to unlock %s\n"), Prog, gr_dbname ());
 			SYSLOG ((LOG_ERR, "failed to unlock %s", gr_dbname ()));
 			/* continue */
@@ -640,7 +650,7 @@ fail_exit (int code)
 	}
 #ifdef	SHADOWGRP
 	if (sgr_locked) {
-		if (sgr_unlock () == 0) {
+		if (sgr_unlock (process_selinux) == 0) {
 			fprintf (stderr, _("%s: failed to unlock %s\n"), Prog, sgr_dbname ());
 			SYSLOG ((LOG_ERR, "failed to unlock %s", sgr_dbname ()));
 			/* continue */
@@ -648,14 +658,14 @@ fail_exit (int code)
 	}
 #endif
 	if (spw_locked) {
-		if (spw_unlock () == 0) {
+		if (spw_unlock (process_selinux) == 0) {
 			fprintf (stderr, _("%s: failed to unlock %s\n"), Prog, spw_dbname ());
 			SYSLOG ((LOG_ERR, "failed to unlock %s", spw_dbname ()));
 			/* continue */
 		}
 	}
 	if (pw_locked) {
-		if (pw_unlock () == 0) {
+		if (pw_unlock (process_selinux) == 0) {
 			fprintf (stderr, _("%s: failed to unlock %s\n"), Prog, pw_dbname ());
 			SYSLOG ((LOG_ERR, "failed to unlock %s", pw_dbname ()));
 			/* continue */
@@ -663,14 +673,14 @@ fail_exit (int code)
 	}
 #ifdef ENABLE_SUBIDS
 	if (sub_uid_locked) {
-		if (sub_uid_unlock () == 0) {
+		if (sub_uid_unlock (process_selinux) == 0) {
 			fprintf (stderr, _("%s: failed to unlock %s\n"), Prog, sub_uid_dbname ());
 			SYSLOG ((LOG_ERR, "failed to unlock %s", sub_uid_dbname ()));
 			/* continue */
 		}
 	}
 	if (sub_gid_locked) {
-		if (sub_gid_unlock () == 0) {
+		if (sub_gid_unlock (process_selinux) == 0) {
 			fprintf (stderr, _("%s: failed to unlock %s\n"), Prog, sub_gid_dbname ());
 			SYSLOG ((LOG_ERR, "failed to unlock %s", sub_gid_dbname ()));
 			/* continue */
@@ -679,7 +689,7 @@ fail_exit (int code)
 #endif				/* ENABLE_SUBIDS */
 
 #ifdef WITH_AUDIT
-	audit_logger (AUDIT_USER_MGMT, Prog,
+	audit_logger (AUDIT_USER_MGMT,
 	              "modify-account",
 	              user_name, AUDIT_NO_ID, SHADOW_AUDIT_FAILURE);
 #endif
@@ -688,7 +698,7 @@ fail_exit (int code)
 
 
 static void
-update_group_file(void)
+update_group_file(bool process_selinux)
 {
 	const struct group  *grp;
 
@@ -696,13 +706,13 @@ update_group_file(void)
 	 * Scan through the entire group file looking for the groups that
 	 * the user is a member of.
 	 */
-	while ((grp = gr_next()) != NULL)
-		update_group(grp);
+	while (NULL != (grp = gr_next()))
+		update_group(grp, process_selinux);
 }
 
 
 static void
-update_group(const struct group *grp)
+update_group(const struct group *grp, bool process_selinux)
 {
 	bool          changed;
 	bool          is_member;
@@ -735,7 +745,7 @@ update_group(const struct group *grp)
 		fprintf (stderr,
 			 _("%s: Out of memory. Cannot update %s.\n"),
 			 Prog, gr_dbname ());
-		fail_exit (E_GRP_UPDATE);
+		fail_exit (E_GRP_UPDATE, process_selinux);
 	}
 
 	if (was_member) {
@@ -804,7 +814,7 @@ update_group(const struct group *grp)
 			 _("%s: failed to prepare the new %s entry '%s'\n"),
 			 Prog, gr_dbname (), ngrp->gr_name);
 		SYSLOG ((LOG_WARN, "failed to prepare the new %s entry '%s'", gr_dbname (), ngrp->gr_name));
-		fail_exit (E_GRP_UPDATE);
+		fail_exit (E_GRP_UPDATE, process_selinux);
 	}
 
 free_ngrp:
@@ -814,7 +824,7 @@ free_ngrp:
 
 #ifdef SHADOWGRP
 static void
-update_gshadow_file(void)
+update_gshadow_file(bool process_selinux)
 {
 	const struct sgrp  *sgrp;
 
@@ -822,15 +832,15 @@ update_gshadow_file(void)
 	 * Scan through the entire shadow group file looking for the groups
 	 * that the user is a member of.
 	 */
-	while ((sgrp = sgr_next()) != NULL)
-		update_gshadow(sgrp);
+	while (NULL != (sgrp = sgr_next()))
+		update_gshadow(sgrp, process_selinux);
 }
 #endif				/* SHADOWGRP */
 
 
 #ifdef SHADOWGRP
 static void
-update_gshadow(const struct sgrp *sgrp)
+update_gshadow(const struct sgrp *sgrp, bool process_selinux)
 {
 	bool         changed;
 	bool         is_member;
@@ -873,7 +883,7 @@ update_gshadow(const struct sgrp *sgrp)
 		fprintf (stderr,
 			 _("%s: Out of memory. Cannot update %s.\n"),
 			 Prog, sgr_dbname ());
-		fail_exit (E_GRP_UPDATE);
+		fail_exit (E_GRP_UPDATE, process_selinux);
 	}
 
 	if (was_admin && lflg) {
@@ -960,7 +970,7 @@ update_gshadow(const struct sgrp *sgrp)
 			 Prog, sgr_dbname (), nsgrp->sg_namp);
 		SYSLOG ((LOG_WARN, "failed to prepare the new %s entry '%s'",
 			 sgr_dbname (), nsgrp->sg_namp));
-		fail_exit (E_GRP_UPDATE);
+		fail_exit (E_GRP_UPDATE, process_selinux);
 	}
 
 free_nsgrp:
@@ -975,12 +985,12 @@ free_nsgrp:
  *	grp_update() takes the secondary group set given in user_groups and
  *	adds the user to each group given by that set.
  */
-static void grp_update (void)
+static void grp_update (bool process_selinux)
 {
-	update_group_file();
+	update_group_file(process_selinux);
 #ifdef SHADOWGRP
 	if (is_shadow_grp) {
-		update_gshadow_file();
+		update_gshadow_file(process_selinux);
 	}
 #endif
 }
@@ -993,7 +1003,7 @@ static void grp_update (void)
  *	are checked for sanity.
  */
 static void
-process_flags(int argc, char **argv)
+process_flags(int argc, char **argv, struct option_flags *flags)
 {
 	struct stat st;
 	bool anyflag = false;
@@ -1127,7 +1137,7 @@ process_flags(int argc, char **argv)
 				/*@notreached@*/break;
 			case 'l':
 				if (!is_valid_user_name(optarg)) {
-					if (errno == EINVAL) {
+					if (errno == EILSEQ) {
 						fprintf(stderr,
 						        _("%s: invalid user name '%s': use --badname to ignore\n"),
 						        Prog, optarg);
@@ -1158,8 +1168,10 @@ process_flags(int argc, char **argv)
 				rflg = true;
 				break;
 			case 'R': /* no-op, handled in process_root_flag () */
+				flags->chroot = true;
 				break;
 			case 'P': /* no-op, handled in process_prefix_flag () */
+				flags->prefix = true;
 				break;
 			case 's':
 				if (   ( !VALID (optarg) )
@@ -1225,7 +1237,7 @@ process_flags(int argc, char **argv)
 					exit(E_BAD_ARG);
 				}
 				wflg = true;
-                break;
+				break;
 			case 'W':
 				if (prepend_range (optarg, &del_sub_gids) == 0) {
 					fprintf (stderr,
@@ -1376,49 +1388,6 @@ process_flags(int argc, char **argv)
 	}
 #endif				/* WITH_SELINUX */
 
-	if (user_newid == user_id) {
-		uflg = false;
-		oflg = false;
-	}
-	if (user_newgid == user_gid) {
-		gflg = false;
-	}
-	if (   (NULL != user_newshell)
-	    && streq(user_newshell, user_shell)) {
-		sflg = false;
-	}
-	if (streq(user_newname, user_name)) {
-		lflg = false;
-	}
-	if (user_newinactive == user_inactive) {
-		fflg = false;
-	}
-	if (user_newexpire == user_expire) {
-		eflg = false;
-	}
-	if (   (NULL != user_newhome)
-	    && streq(user_newhome, user_home)) {
-		dflg = false;
-		mflg = false;
-	}
-	if (   (NULL != user_newcomment)
-	    && streq(user_newcomment, user_comment)) {
-		cflg = false;
-	}
-
-	if (!(Uflg || uflg || sflg || pflg || mflg || Lflg ||
-	      lflg || Gflg || gflg || fflg || eflg || dflg || cflg
-#ifdef ENABLE_SUBIDS
-	      || vflg || Vflg || wflg || Wflg
-#endif				/* ENABLE_SUBIDS */
-#ifdef WITH_SELINUX
-	      || Zflg
-#endif				/* WITH_SELINUX */
-	)) {
-		fprintf (stdout, _("%s: no changes\n"), Prog);
-		exit (E_SUCCESS);
-	}
-
 	if (!is_shadow_pwd && (eflg || fflg)) {
 		fprintf (stderr,
 		         _("%s: shadow passwords required for -e and -f\n"),
@@ -1467,51 +1436,55 @@ process_flags(int argc, char **argv)
  *	close_files() closes all of the files that were opened for this new
  *	user. This causes any modified entries to be written out.
  */
-static void close_files (void)
+static void close_files(const struct option_flags *flags)
 {
-	if (pw_close () == 0) {
+	bool process_selinux;
+
+	process_selinux = !flags->chroot && !flags->prefix;
+
+	if (pw_close (process_selinux) == 0) {
 		fprintf (stderr,
 		         _("%s: failure while writing changes to %s\n"),
 		         Prog, pw_dbname ());
 		SYSLOG ((LOG_ERR, "failure while writing changes to %s", pw_dbname ()));
-		fail_exit (E_PW_UPDATE);
+		fail_exit (E_PW_UPDATE, process_selinux);
 	}
-	if (is_shadow_pwd && (spw_close () == 0)) {
+	if (is_shadow_pwd && (spw_close (process_selinux) == 0)) {
 		fprintf (stderr,
 		         _("%s: failure while writing changes to %s\n"),
 		         Prog, spw_dbname ());
 		SYSLOG ((LOG_ERR,
 		         "failure while writing changes to %s",
 		         spw_dbname ()));
-		fail_exit (E_PW_UPDATE);
+		fail_exit (E_PW_UPDATE, process_selinux);
 	}
 
 	if (Gflg || lflg) {
-		if (gr_close () == 0) {
+		if (gr_close (process_selinux) == 0) {
 			fprintf (stderr,
 			         _("%s: failure while writing changes to %s\n"),
 			         Prog, gr_dbname ());
 			SYSLOG ((LOG_ERR,
 			         "failure while writing changes to %s",
 			         gr_dbname ()));
-			fail_exit (E_GRP_UPDATE);
+			fail_exit (E_GRP_UPDATE, process_selinux);
 		}
 #ifdef SHADOWGRP
 		if (is_shadow_grp) {
-			if (sgr_close () == 0) {
+			if (sgr_close (process_selinux) == 0) {
 				fprintf (stderr,
 				         _("%s: failure while writing changes to %s\n"),
 				         Prog, sgr_dbname ());
 				SYSLOG ((LOG_ERR,
 				         "failure while writing changes to %s",
 				         sgr_dbname ()));
-				fail_exit (E_GRP_UPDATE);
+				fail_exit (E_GRP_UPDATE, process_selinux);
 			}
 		}
 #endif
 #ifdef SHADOWGRP
 		if (is_shadow_grp) {
-			if (sgr_unlock () == 0) {
+			if (sgr_unlock (process_selinux) == 0) {
 				fprintf (stderr,
 				         _("%s: failed to unlock %s\n"),
 				         Prog, sgr_dbname ());
@@ -1522,7 +1495,7 @@ static void close_files (void)
 			}
 		}
 #endif
-		if (gr_unlock () == 0) {
+		if (gr_unlock (process_selinux) == 0) {
 			fprintf (stderr,
 			         _("%s: failed to unlock %s\n"),
 			         Prog, gr_dbname ());
@@ -1534,7 +1507,7 @@ static void close_files (void)
 	}
 
 	if (is_shadow_pwd) {
-		if (spw_unlock () == 0) {
+		if (spw_unlock (process_selinux) == 0) {
 			fprintf (stderr,
 			         _("%s: failed to unlock %s\n"),
 			         Prog, spw_dbname ());
@@ -1544,7 +1517,7 @@ static void close_files (void)
 			/* continue */
 		}
 	}
-	if (pw_unlock () == 0) {
+	if (pw_unlock (process_selinux) == 0) {
 		fprintf (stderr,
 		         _("%s: failed to unlock %s\n"),
 		         Prog, pw_dbname ());
@@ -1561,12 +1534,12 @@ static void close_files (void)
 
 #ifdef ENABLE_SUBIDS
 	if (vflg || Vflg) {
-		if (sub_uid_close () == 0) {
+		if (sub_uid_close (process_selinux) == 0) {
 			fprintf (stderr, _("%s: failure while writing changes to %s\n"), Prog, sub_uid_dbname ());
 			SYSLOG ((LOG_ERR, "failure while writing changes to %s", sub_uid_dbname ()));
-			fail_exit (E_SUB_UID_UPDATE);
+			fail_exit (E_SUB_UID_UPDATE, process_selinux);
 		}
-		if (sub_uid_unlock () == 0) {
+		if (sub_uid_unlock (process_selinux) == 0) {
 			fprintf (stderr, _("%s: failed to unlock %s\n"), Prog, sub_uid_dbname ());
 			SYSLOG ((LOG_ERR, "failed to unlock %s", sub_uid_dbname ()));
 			/* continue */
@@ -1574,12 +1547,12 @@ static void close_files (void)
 		sub_uid_locked = false;
 	}
 	if (wflg || Wflg) {
-		if (sub_gid_close () == 0) {
+		if (sub_gid_close (process_selinux) == 0) {
 			fprintf (stderr, _("%s: failure while writing changes to %s\n"), Prog, sub_gid_dbname ());
 			SYSLOG ((LOG_ERR, "failure while writing changes to %s", sub_gid_dbname ()));
-			fail_exit (E_SUB_GID_UPDATE);
+			fail_exit (E_SUB_GID_UPDATE, process_selinux);
 		}
-		if (sub_gid_unlock () == 0) {
+		if (sub_gid_unlock (process_selinux) == 0) {
 			fprintf (stderr, _("%s: failed to unlock %s\n"), Prog, sub_gid_dbname ());
 			SYSLOG ((LOG_ERR, "failed to unlock %s", sub_gid_dbname ()));
 			/* continue */
@@ -1604,33 +1577,33 @@ static void close_files (void)
  *
  *	open_files() opens the two password files.
  */
-static void open_files (void)
+static void open_files (bool process_selinux)
 {
 	if (pw_lock () == 0) {
 		fprintf (stderr,
 		         _("%s: cannot lock %s; try again later.\n"),
 		         Prog, pw_dbname ());
-		fail_exit (E_PW_UPDATE);
+		fail_exit (E_PW_UPDATE, process_selinux);
 	}
 	pw_locked = true;
 	if (pw_open (O_CREAT | O_RDWR) == 0) {
 		fprintf (stderr,
 		         _("%s: cannot open %s\n"),
 		         Prog, pw_dbname ());
-		fail_exit (E_PW_UPDATE);
+		fail_exit (E_PW_UPDATE, process_selinux);
 	}
 	if (is_shadow_pwd && (spw_lock () == 0)) {
 		fprintf (stderr,
 		         _("%s: cannot lock %s; try again later.\n"),
 		         Prog, spw_dbname ());
-		fail_exit (E_PW_UPDATE);
+		fail_exit (E_PW_UPDATE, process_selinux);
 	}
 	spw_locked = true;
 	if (is_shadow_pwd && (spw_open (O_CREAT | O_RDWR) == 0)) {
 		fprintf (stderr,
 		         _("%s: cannot open %s\n"),
 		         Prog, spw_dbname ());
-		fail_exit (E_PW_UPDATE);
+		fail_exit (E_PW_UPDATE, process_selinux);
 	}
 
 	if (Gflg || lflg) {
@@ -1642,28 +1615,28 @@ static void open_files (void)
 			fprintf (stderr,
 			         _("%s: cannot lock %s; try again later.\n"),
 			         Prog, gr_dbname ());
-			fail_exit (E_GRP_UPDATE);
+			fail_exit (E_GRP_UPDATE, process_selinux);
 		}
 		gr_locked = true;
 		if (gr_open (O_CREAT | O_RDWR) == 0) {
 			fprintf (stderr,
 			         _("%s: cannot open %s\n"),
 			         Prog, gr_dbname ());
-			fail_exit (E_GRP_UPDATE);
+			fail_exit (E_GRP_UPDATE, process_selinux);
 		}
 #ifdef SHADOWGRP
 		if (is_shadow_grp && (sgr_lock () == 0)) {
 			fprintf (stderr,
 			         _("%s: cannot lock %s; try again later.\n"),
 			         Prog, sgr_dbname ());
-			fail_exit (E_GRP_UPDATE);
+			fail_exit (E_GRP_UPDATE, process_selinux);
 		}
 		sgr_locked = true;
 		if (is_shadow_grp && (sgr_open (O_CREAT | O_RDWR) == 0)) {
 			fprintf (stderr,
 			         _("%s: cannot open %s\n"),
 			         Prog, sgr_dbname ());
-			fail_exit (E_GRP_UPDATE);
+			fail_exit (E_GRP_UPDATE, process_selinux);
 		}
 #endif
 	}
@@ -1673,14 +1646,14 @@ static void open_files (void)
 			fprintf (stderr,
 			         _("%s: cannot lock %s; try again later.\n"),
 			         Prog, sub_uid_dbname ());
-			fail_exit (E_SUB_UID_UPDATE);
+			fail_exit (E_SUB_UID_UPDATE, process_selinux);
 		}
 		sub_uid_locked = true;
 		if (sub_uid_open (O_CREAT | O_RDWR) == 0) {
 			fprintf (stderr,
 			         _("%s: cannot open %s\n"),
 			         Prog, sub_uid_dbname ());
-			fail_exit (E_SUB_UID_UPDATE);
+			fail_exit (E_SUB_UID_UPDATE, process_selinux);
 		}
 	}
 	if (wflg || Wflg) {
@@ -1688,14 +1661,14 @@ static void open_files (void)
 			fprintf (stderr,
 			         _("%s: cannot lock %s; try again later.\n"),
 			         Prog, sub_gid_dbname ());
-			fail_exit (E_SUB_GID_UPDATE);
+			fail_exit (E_SUB_GID_UPDATE, process_selinux);
 		}
 		sub_gid_locked = true;
 		if (sub_gid_open (O_CREAT | O_RDWR) == 0) {
 			fprintf (stderr,
 			         _("%s: cannot open %s\n"),
 			         Prog, sub_gid_dbname ());
-			fail_exit (E_SUB_GID_UPDATE);
+			fail_exit (E_SUB_GID_UPDATE, process_selinux);
 		}
 	}
 #endif				/* ENABLE_SUBIDS */
@@ -1707,13 +1680,15 @@ static void open_files (void)
  *	usr_update() creates the password file entries for this user and
  *	will update the group entries if required.
  */
-static void usr_update (void)
+static void usr_update(const struct option_flags *flags)
 {
 	struct passwd pwent;
 	const struct passwd *pwd;
-
 	struct spwd spent;
 	const struct spwd *spwd = NULL;
+	bool process_selinux;
+
+	process_selinux = !flags->chroot && !flags->prefix;
 
 	/*
 	 * Locate the entry in /etc/passwd, which MUST exist.
@@ -1723,10 +1698,10 @@ static void usr_update (void)
 		fprintf (stderr,
 		         _("%s: user '%s' does not exist in %s\n"),
 		         Prog, user_name, pw_dbname ());
-		fail_exit (E_NOTFOUND);
+		fail_exit (E_NOTFOUND, process_selinux);
 	}
 	pwent = *pwd;
-	new_pwent (&pwent);
+	new_pwent (&pwent, process_selinux);
 
 
 	/* If the shadow file does not exist, it won't be created */
@@ -1735,7 +1710,7 @@ static void usr_update (void)
 		if (NULL != spwd) {
 			/* Update the shadow entry if it exists */
 			spent = *spwd;
-			new_spent (&spent);
+			new_spent (&spent, process_selinux);
 		} else if (   (    pflg
 		               && streq(pwent.pw_passwd, SHADOW_PASSWD_STRING))
 		           || eflg || fflg) {
@@ -1745,7 +1720,7 @@ static void usr_update (void)
 			 *    a shadowed password
 			 *  + aging information is requested
 			 */
-			bzero(&spent, sizeof spent);
+			bzero(&spent, sizeof(spent));
 			spent.sp_namp   = user_name;
 
 			/* The user explicitly asked for a shadow feature.
@@ -1766,7 +1741,7 @@ static void usr_update (void)
 			spent.sp_inact  = -1;
 			spent.sp_expire = -1;
 			spent.sp_flag   = SHADOW_SP_FLAG_UNSET;
-			new_spent (&spent);
+			new_spent (&spent, process_selinux);
 			spwd = &spent; /* entry needs to be committed */
 		}
 	}
@@ -1777,13 +1752,13 @@ static void usr_update (void)
 			fprintf (stderr,
 			         _("%s: failed to prepare the new %s entry '%s'\n"),
 			         Prog, pw_dbname (), pwent.pw_name);
-			fail_exit (E_PW_UPDATE);
+			fail_exit (E_PW_UPDATE, process_selinux);
 		}
 		if (lflg && (pw_remove (user_name) == 0)) {
 			fprintf (stderr,
 			         _("%s: cannot remove entry '%s' from %s\n"),
 			         Prog, user_name, pw_dbname ());
-			fail_exit (E_PW_UPDATE);
+			fail_exit (E_PW_UPDATE, process_selinux);
 		}
 	}
 	if ((NULL != spwd) && (lflg || eflg || fflg || pflg || Lflg || Uflg)) {
@@ -1791,13 +1766,13 @@ static void usr_update (void)
 			fprintf (stderr,
 			         _("%s: failed to prepare the new %s entry '%s'\n"),
 			         Prog, spw_dbname (), spent.sp_namp);
-			fail_exit (E_PW_UPDATE);
+			fail_exit (E_PW_UPDATE, process_selinux);
 		}
 		if (lflg && (spw_remove (user_name) == 0)) {
 			fprintf (stderr,
 			         _("%s: cannot remove entry '%s' from %s\n"),
 			         Prog, user_name, spw_dbname ());
-			fail_exit (E_PW_UPDATE);
+			fail_exit (E_PW_UPDATE, process_selinux);
 		}
 	}
 }
@@ -1808,7 +1783,7 @@ static void usr_update (void)
  *	move_home() moves the user's home directory to a new location. The
  *	files will be copied if the directory cannot simply be renamed.
  */
-static void move_home (void)
+static void move_home (bool process_selinux)
 {
 	struct stat sb;
 
@@ -1820,7 +1795,7 @@ static void move_home (void)
 		fprintf (stderr,
 		         _("%s: directory %s exists\n"),
 		         Prog, user_newhome);
-		fail_exit (E_HOMEDIR);
+		fail_exit (E_HOMEDIR, process_selinux);
 	}
 
 	if (stat (prefix_user_home, &sb) == 0) {
@@ -1834,12 +1809,12 @@ static void move_home (void)
 			           "not a directory. It is not removed and no "
 			           "home directories are created.\n"),
 			         Prog, user_home);
-			fail_exit (E_HOMEDIR);
+			fail_exit (E_HOMEDIR, process_selinux);
 		}
 
 #ifdef WITH_AUDIT
 		if (uflg || gflg) {
-			audit_logger (AUDIT_USER_MGMT, Prog,
+			audit_logger (AUDIT_USER_MGMT,
 				      "updating-home-dir-owner",
 				      user_newname, user_newid, 1);
 		}
@@ -1855,10 +1830,10 @@ static void move_home (void)
 				fprintf (stderr,
 				         _("%s: Failed to change ownership of the home directory"),
 				         Prog);
-				fail_exit (E_HOMEDIR);
+				fail_exit (E_HOMEDIR, process_selinux);
 			}
 #ifdef WITH_AUDIT
-			audit_logger (AUDIT_USER_MGMT, Prog,
+			audit_logger (AUDIT_USER_MGMT,
 			              "moving-home-dir",
 			              user_newname, user_newid, 1);
 #endif
@@ -1870,7 +1845,7 @@ static void move_home (void)
 					fprintf (stderr,
 					        _("%s: error: cannot move subvolume from %s to %s - different device\n"),
 					        Prog, prefix_user_home, prefix_user_newhome);
-					fail_exit (E_HOMEDIR);
+					fail_exit (E_HOMEDIR, process_selinux);
 				}
 #endif
 
@@ -1887,7 +1862,6 @@ static void move_home (void)
 					}
 #ifdef WITH_AUDIT
 					audit_logger (AUDIT_USER_MGMT,
-					              Prog,
 					              "moving-home-dir",
 					              user_newname,
 					              user_newid,
@@ -1901,7 +1875,7 @@ static void move_home (void)
 			fprintf (stderr,
 			         _("%s: cannot rename directory %s to %s\n"),
 			         Prog, prefix_user_home, prefix_user_newhome);
-			fail_exit (E_HOMEDIR);
+			fail_exit (E_HOMEDIR, process_selinux);
 		}
 	} else {
 		fprintf (stderr,
@@ -1923,11 +1897,11 @@ static void update_lastlog (void)
 {
 	struct lastlog ll;
 	int fd;
-	off_t off_uid = (off_t) user_id * sizeof ll;
-	off_t off_newuid = (off_t) user_newid * sizeof ll;
+	off_t off_uid = (off_t) user_id * sizeof(ll);
+	off_t off_newuid = (off_t) user_newid * sizeof(ll);
 	uid_t max_uid;
 
-	if (access (LASTLOG_FILE, F_OK) != 0) {
+	if (access(_PATH_LASTLOG, F_OK) != 0) {
 		return;
 	}
 
@@ -1937,24 +1911,24 @@ static void update_lastlog (void)
 		return;
 	}
 
-	fd = open (LASTLOG_FILE, O_RDWR);
+	fd = open(_PATH_LASTLOG, O_RDWR);
 
 	if (-1 == fd) {
 		fprintf (stderr,
 		         _("%s: failed to copy the lastlog entry of user %lu to user %lu: %s\n"),
-		         Prog, (unsigned long) user_id, (unsigned long) user_newid, strerror (errno));
+		        Prog, (unsigned long) user_id, (unsigned long) user_newid, strerrno());
 		return;
 	}
 
 	if (   (lseek (fd, off_uid, SEEK_SET) == off_uid)
-	    && (read (fd, &ll, sizeof ll) == (ssize_t) sizeof ll)) {
+	    && (read(fd, &ll, sizeof(ll)) == (ssize_t) sizeof(ll))) {
 		/* Copy the old entry to its new location */
 		if (   (lseek (fd, off_newuid, SEEK_SET) != off_newuid)
-		    || (write_full(fd, &ll, sizeof ll) == -1)
+		    || (write_full(fd, &ll, sizeof(ll)) == -1)
 		    || (fsync (fd) != 0)) {
 			fprintf (stderr,
 			         _("%s: failed to copy the lastlog entry of user %lu to user %lu: %s\n"),
-			         Prog, (unsigned long) user_id, (unsigned long) user_newid, strerror (errno));
+			        Prog, (unsigned long) user_id, (unsigned long) user_newid, strerrno());
 		}
 	} else {
 		/* Assume lseek or read failed because there is
@@ -1962,15 +1936,15 @@ static void update_lastlog (void)
 
 		/* Check if the new UID already has an entry */
 		if (   (lseek (fd, off_newuid, SEEK_SET) == off_newuid)
-		    && (read (fd, &ll, sizeof ll) == (ssize_t) sizeof ll)) {
+		    && (read(fd, &ll, sizeof(ll)) == (ssize_t) sizeof(ll))) {
 			/* Reset the new uid's lastlog entry */
-			memzero (&ll, sizeof (ll));
+			memzero(&ll, sizeof(ll));
 			if (   (lseek (fd, off_newuid, SEEK_SET) != off_newuid)
-			    || (write_full(fd, &ll, sizeof ll) == -1)
+			    || (write_full(fd, &ll, sizeof(ll)) == -1)
 			    || (fsync (fd) != 0)) {
 				fprintf (stderr,
 				         _("%s: failed to copy the lastlog entry of user %lu to user %lu: %s\n"),
-				         Prog, (unsigned long) user_id, (unsigned long) user_newid, strerror (errno));
+				        Prog, (unsigned long) user_id, (unsigned long) user_newid, strerrno());
 			}
 		}
 	}
@@ -1978,7 +1952,7 @@ static void update_lastlog (void)
 	if (close (fd) != 0 && errno != EINTR) {
 		fprintf (stderr,
 		         _("%s: failed to copy the lastlog entry of user %ju to user %ju: %s\n"),
-		         Prog, (uintmax_t) user_id, (uintmax_t) user_newid, strerror (errno));
+		        Prog, (uintmax_t) user_id, (uintmax_t) user_newid, strerrno());
 	}
 }
 #endif /* ENABLE_LASTLOG */
@@ -1994,8 +1968,8 @@ static void update_faillog (void)
 {
 	struct faillog fl;
 	int fd;
-	off_t off_uid = (off_t) user_id * sizeof fl;
-	off_t off_newuid = (off_t) user_newid * sizeof fl;
+	off_t off_uid = (off_t) user_id * sizeof(fl);
+	off_t off_newuid = (off_t) user_newid * sizeof(fl);
 
 	if (access (FAILLOG_FILE, F_OK) != 0) {
 		return;
@@ -2006,19 +1980,19 @@ static void update_faillog (void)
 	if (-1 == fd) {
 		fprintf (stderr,
 		         _("%s: failed to copy the faillog entry of user %lu to user %lu: %s\n"),
-		         Prog, (unsigned long) user_id, (unsigned long) user_newid, strerror (errno));
+		        Prog, (unsigned long) user_id, (unsigned long) user_newid, strerrno());
 		return;
 	}
 
 	if (   (lseek (fd, off_uid, SEEK_SET) == off_uid)
-	    && (read (fd, &fl, sizeof fl) == (ssize_t) sizeof fl)) {
+	    && (read(fd, &fl, sizeof(fl)) == (ssize_t) sizeof(fl))) {
 		/* Copy the old entry to its new location */
 		if (   (lseek (fd, off_newuid, SEEK_SET) != off_newuid)
-		    || (write_full(fd, &fl, sizeof fl) == -1)
+		    || (write_full(fd, &fl, sizeof(fl)) == -1)
 		    || (fsync (fd) != 0)) {
 			fprintf (stderr,
 			         _("%s: failed to copy the faillog entry of user %lu to user %lu: %s\n"),
-			         Prog, (unsigned long) user_id, (unsigned long) user_newid, strerror (errno));
+			        Prog, (unsigned long) user_id, (unsigned long) user_newid, strerrno());
 		}
 	} else {
 		/* Assume lseek or read failed because there is
@@ -2026,15 +2000,15 @@ static void update_faillog (void)
 
 		/* Check if the new UID already has an entry */
 		if (   (lseek (fd, off_newuid, SEEK_SET) == off_newuid)
-		    && (read (fd, &fl, sizeof fl) == (ssize_t) sizeof fl)) {
+		    && (read(fd, &fl, sizeof(fl)) == (ssize_t) sizeof(fl))) {
 			/* Reset the new uid's faillog entry */
-			memzero (&fl, sizeof (fl));
+			memzero(&fl, sizeof(fl));
 			if (   (lseek (fd, off_newuid, SEEK_SET) != off_newuid)
-			    || (write_full(fd, &fl, sizeof fl) == -1))
+			    || (write_full(fd, &fl, sizeof(fl)) == -1))
 			{
 				fprintf (stderr,
 				         _("%s: failed to copy the faillog entry of user %lu to user %lu: %s\n"),
-				         Prog, (unsigned long) user_id, (unsigned long) user_newid, strerror (errno));
+				        Prog, (unsigned long) user_id, (unsigned long) user_newid, strerrno());
 			}
 		}
 	}
@@ -2042,7 +2016,7 @@ static void update_faillog (void)
 	if (close (fd) != 0 && errno != EINTR) {
 		fprintf (stderr,
 		         _("%s: failed to copy the faillog entry of user %ju to user %ju: %s\n"),
-		         Prog, (uintmax_t) user_id, (uintmax_t) user_newid, strerror (errno));
+		        Prog, (uintmax_t) user_id, (uintmax_t) user_newid, strerrno());
 	}
 }
 
@@ -2112,7 +2086,7 @@ static void move_mailbox (void)
 		}
 #ifdef WITH_AUDIT
 		else {
-			audit_logger (AUDIT_USER_MGMT, Prog,
+			audit_logger (AUDIT_USER_MGMT,
 			              "updating-mail-file-owner",
 			              user_newname, user_newid, 1);
 		}
@@ -2136,7 +2110,7 @@ static void move_mailbox (void)
 		}
 #ifdef WITH_AUDIT
 		else {
-			audit_logger (AUDIT_USER_MGMT, Prog,
+			audit_logger (AUDIT_USER_MGMT,
 			              "updating-mail-file-name",
 			              user_newname, user_newid, 1);
 		}
@@ -2160,6 +2134,8 @@ int main (int argc, char **argv)
 	int retval;
 #endif				/* USE_PAM */
 #endif				/* ACCT_TOOLS_SETUID */
+	struct option_flags  flags = {.chroot = false, .prefix = false};
+	bool process_selinux;
 
 	log_set_progname(Prog);
 	log_set_logfd(stderr);
@@ -2177,7 +2153,7 @@ int main (int argc, char **argv)
 #endif
 
 	sys_ngroups = sysconf (_SC_NGROUPS_MAX);
-	user_groups = XMALLOC(sys_ngroups + 1, char *);
+	user_groups = xmalloc_T(sys_ngroups + 1, char *);
 	user_groups[0] = NULL;
 
 	is_shadow_pwd = spw_file_present ();
@@ -2185,11 +2161,12 @@ int main (int argc, char **argv)
 	is_shadow_grp = sgr_file_present ();
 #endif
 #ifdef ENABLE_SUBIDS
-	is_sub_uid = sub_uid_file_present ();
-	is_sub_gid = sub_gid_file_present ();
+	is_sub_uid = want_subuid_file() && sub_uid_file_present();
+	is_sub_gid = want_subgid_file() && sub_gid_file_present();
 #endif				/* ENABLE_SUBIDS */
 
-	process_flags (argc, argv);
+	process_flags (argc, argv, &flags);
+	process_selinux = !flags.chroot && !flags.prefix;
 
 	/*
 	 * The home directory, the username and the user's UID should not
@@ -2251,13 +2228,13 @@ int main (int argc, char **argv)
 	 * Do the hard stuff - open the files, change the user entries,
 	 * change the home directory, then close and update the files.
 	 */
-	open_files ();
+	open_files (process_selinux);
 	if (   cflg || dflg || eflg || fflg || gflg || Lflg || lflg || pflg
 	    || sflg || uflg || Uflg) {
-		usr_update ();
+		usr_update (&flags);
 	}
 	if (Gflg || lflg) {
-		grp_update ();
+		grp_update (process_selinux);
 	}
 #ifdef ENABLE_SUBIDS
 	if (Vflg) {
@@ -2273,7 +2250,7 @@ int main (int argc, char **argv)
 				        (uintmax_t) ptr->range.first,
 				        (uintmax_t) ptr->range.last,
 				        sub_uid_dbname());
-				fail_exit (E_SUB_UID_UPDATE);
+				fail_exit (E_SUB_UID_UPDATE, process_selinux);
 			}
 		}
 	}
@@ -2290,7 +2267,7 @@ int main (int argc, char **argv)
 				        (uintmax_t) ptr->range.first,
 				        (uintmax_t) ptr->range.last,
 				        sub_uid_dbname());
-				fail_exit (E_SUB_UID_UPDATE);
+				fail_exit (E_SUB_UID_UPDATE, process_selinux);
 			}
 		}
 	}
@@ -2307,7 +2284,7 @@ int main (int argc, char **argv)
 				        (uintmax_t) ptr->range.first,
 				        (uintmax_t) ptr->range.last,
 				        sub_gid_dbname());
-				fail_exit (E_SUB_GID_UPDATE);
+				fail_exit (E_SUB_GID_UPDATE, process_selinux);
 			}
 		}
 	}
@@ -2324,12 +2301,12 @@ int main (int argc, char **argv)
 				        (uintmax_t) ptr->range.first,
 				        (uintmax_t) ptr->range.last,
 				        sub_gid_dbname());
-				fail_exit (E_SUB_GID_UPDATE);
+				fail_exit (E_SUB_GID_UPDATE, process_selinux);
 			}
 		}
 	}
 #endif				/* ENABLE_SUBIDS */
-	close_files ();
+	close_files (&flags);
 
 #ifdef WITH_TCB
 	if (   (lflg || uflg)
@@ -2350,12 +2327,12 @@ int main (int argc, char **argv)
 				         _("%s: warning: the user name %s to %s SELinux user mapping failed.\n"),
 				         Prog, user_name, user_selinux);
 #ifdef WITH_AUDIT
-				audit_logger (AUDIT_ROLE_ASSIGN, Prog,
+				audit_logger (AUDIT_ROLE_ASSIGN,
 				              "changing-selinux-user-mapping ",
 				              user_name, user_id,
 				              SHADOW_AUDIT_FAILURE);
 #endif				/* WITH_AUDIT */
-				fail_exit (E_SE_UPDATE);
+				fail_exit (E_SE_UPDATE, process_selinux);
 			}
 		} else {
 			if (del_seuser (user_name) != 0) {
@@ -2363,19 +2340,19 @@ int main (int argc, char **argv)
 				         _("%s: warning: the user name %s to SELinux user mapping removal failed.\n"),
 				         Prog, user_name);
 #ifdef WITH_AUDIT
-				audit_logger (AUDIT_ROLE_REMOVE, Prog,
+				audit_logger (AUDIT_ROLE_REMOVE,
 				              "delete-selinux-user-mapping",
 				              user_name, user_id,
 				              SHADOW_AUDIT_FAILURE);
 #endif				/* WITH_AUDIT */
-				fail_exit (E_SE_UPDATE);
+				fail_exit (E_SE_UPDATE, process_selinux);
 			}
 		}
 	}
 #endif				/* WITH_SELINUX */
 
 	if (mflg) {
-		move_home ();
+		move_home (process_selinux);
 	}
 
 #ifndef NO_MOVE_MAILBOX
@@ -2407,7 +2384,7 @@ int main (int argc, char **argv)
 			 */
 #ifdef WITH_AUDIT
 			if (uflg || gflg) {
-				audit_logger (AUDIT_USER_MGMT, Prog,
+				audit_logger (AUDIT_USER_MGMT,
 					      "updating-home-dir-owner",
 					      user_newname, user_newid, 1);
 			}
@@ -2420,7 +2397,7 @@ int main (int argc, char **argv)
 				fprintf (stderr,
 				         _("%s: Failed to change ownership of the home directory"),
 				         Prog);
-				fail_exit (E_HOMEDIR);
+				fail_exit (E_HOMEDIR, process_selinux);
 			}
 		}
 	}
